<?php
    class ProtectRemoteResult {
        public $is_success = false;
        public $ip_list = [];
    }

    class ProtectRemote {

        static function pr_start() {
            self::pr_update_htaccess();
        }

        static function pr_get_white_ip_list($prData) {

            $result = new ProtectRemoteResult();
            $settingOptions = new ProtectRemoteSettingOptions();
            
            $args = array(
                'method' => 'GET',
                'headers' => array(
                    'Content-Type' => 'application/json',
                    'X-Token' => $settingOptions->getToken(),
                    'X-AppVersion' => ProtectRemoteVariables::APP_VERSION
                )
            );
    
            $response = wp_remote_get(ProtectRemoteVariables::get_pr_rules_url(), $args);
            ProtectRemoteLog::pr_log_info("Step-1.1 Request to protect remote. URL: ".ProtectRemoteVariables::get_pr_rules_url());
            $response_code = wp_remote_retrieve_response_code($response);
            
            if ($response_code === 200) {

                ProtectRemoteLog::pr_log_info("Step-1.2 Response from protect remote. Status Code: ".$response_code);
                $response_body = wp_remote_retrieve_body($response);

                $obj = json_decode($response_body);
    
                $ipList = [];
                foreach ($obj->rules as $rule) {
                    foreach ($rule->ipAddressV4List as $ipv4) {
                        $ipList[] = $ipv4;
                    }

                    foreach ($rule->ipAddressV6List as $ipv6) {
                        $ipList[] = $ipv6;
                    }
                }   

                self::pr_set_settings($settingOptions, $prData, $obj->settingItems);
                
                $result->ip_list = $ipList;
                $result->is_success = true;
                ProtectRemoteLog::pr_log_info("Step-1.3 Successfully get white ip list. IP List: ".json_encode($ipList));
            } 
            else 
            {               
                if (is_wp_error($response)) {
                    ProtectRemoteLog::pr_log_error("Step-1.2 Failed request to protect remote. Status Code: ".$response_code. " Response: ".json_encode($response));
                } else {
                    ProtectRemoteLog::pr_log_error("Step-1.3 Undefined error. Status code: ".$response_code);
                }

                ProtectRemoteLog::pr_log_error("Step-1.4 Error code: ". $response_code. " Response: ". json_encode($response));
            } 
    
            return $result;
        }

        static function get_cloudflare_ip_list($url) {

            $settingOptions = new ProtectRemoteSettingOptions();

            $args = array(
                'method' => 'GET',
                'headers' => array(
                    'Content-Type' => 'application/json',
                    'X-Token' => $settingOptions->getToken(),
                    'X-AppVersion' => ProtectRemoteVariables::APP_VERSION
                )
            );
    
            $response = wp_remote_get($url, $args);

            $response_code = wp_remote_retrieve_response_code($response);
            
            $ipList = [];

            if ($response_code == 200) {
                
                $response_body = wp_remote_retrieve_body($response);
                $obj = json_decode($response_body);

                foreach ($obj->ipList as $ip) {
                    $ipList[] = $ip;
                }
                
            } else {
                if (is_wp_error($response)) {
                    error_log("Failed request to protect remote. Status Code: ".$response_code. " Response: ".serialize($response));
                } else {
                    error_log("Undefined error. Status code: ".$response_code);
                }
            }

            return $ipList;
        }

        static function pr_set_settings($settingOptions, $prData, $settingItems) {
            
            $logMode = self::get_settings_item($settingItems, 'LogMode');

            $cloudflareIpV4 = self::get_settings_item($settingItems, 'CloudflareIPv4');
            $cloudflareIpV6 = self::get_settings_item($settingItems, 'CloudflareIPv6');

            $logMode != null ? $settingOptions->setWpLogMode($logMode) : $settingOptions->setWpLogMode(false);

            if ($cloudflareIpV4 != null || $cloudflareIpV6 != null) 
            {
                $lastCloudflareIpV4 = $prData->getCloudflareIpV4();
                $lastCloudflareIpV6 = $prData->getCloudflareIpV6();

                if ($lastCloudflareIpV4 != $cloudflareIpV4 || $lastCloudflareIpV6 != $cloudflareIpV6) {
                    
                    $prData->setCloudflareIpV4($cloudflareIpV4);
                    $prData->setCloudflareIpV6($cloudflareIpV6);

                    $cloudflareIpV4List = self::get_cloudflare_ip_list(ProtectRemoteVariables::get_pr_cloudflare_ipv4_url());
                    $cloudflareIpV6List = self::get_cloudflare_ip_list(ProtectRemoteVariables::get_pr_cloudflare_ipv6_url());

                    $prData->setCloudflareIpV4List($cloudflareIpV4List);
                    $prData->setCloudflareIpV6List($cloudflareIpV6List);
                }

                $settingOptions->setWpCloudflare(true);
            } 
            else 
            {
                $settingOptions->setWpCloudflare(false);
            }

            $settingOptions->saveChanges();
        }

        static function get_settings_item($data, $key) {
            $settingValue = null;

            if ($data == null || !is_array($data) || count($data) == 0) {
                return $settingValue;
            }

            foreach ($data as $item) {
                if ($item->key === $key) {
                    $settingValue = $item->value;
                    break;
                }
            }

            return $settingValue;
        }

        static function send_log_to_pr($events) {
    
            if (empty($events) || !is_array($events)) {
                error_log("Invalid or empty events array.");
                return;
            }

            $settingOptions = new ProtectRemoteSettingOptions();
            $json_data = json_encode($events);

            $args = array(
                'method' => 'POST',
                'headers' => array(
                    'Content-Type' => 'application/json',
                    'X-Token' => $settingOptions->getToken(),
                    'X-AppVersion' => ProtectRemoteVariables::APP_VERSION
                ),
                'body'    => $json_data
            );
    
            $response = wp_remote_post(ProtectRemoteVariables::get_pr_exeption_log_event_url(), $args);

            $response_code = wp_remote_retrieve_response_code($response);
            
            if ($response_code != 200) {
                if (is_wp_error($response)) {
                    error_log("Failed request to protect remote. Status Code: ".$response_code. " Response: ".serialize($response));
                } else {
                    error_log("Undefined error. Status code: ".$response_code);
                }
            } 
        }

        static function pr_failover($prData) {
            $tryCount = $prData->getTryCount();

            $hasBlockInHtaccess = $prData->getHasBlockInHtaccess();

            ProtectRemoteLog::pr_log_info("Step-2.1 Failover. Try count: ".$tryCount. " Has block in htaccess: ".$hasBlockInHtaccess);
            
            if ($tryCount >= ProtectRemoteVariables::MAX_TRY_COUNT && $hasBlockInHtaccess) 
            {
                self::remove_pr_block_from_htaccess_and_save();
                $prData->setHasBlockInHtaccess(false);
                ProtectRemoteLog::pr_log_info("Step-2.2 Successfully removed pr block from htaccess.");
            } 
            else if ($tryCount < ProtectRemoteVariables::MAX_TRY_COUNT) 
            {
                $prData->setTryCount($tryCount + 1);
                ProtectRemoteLog::pr_log_info("Step-2.3 Try count less than max try count. Try count: ".$tryCount);
            }

            self::pr_data_set($prData); //After that, there is no prdata update flowing ends

            ProtectRemoteLog::pr_log_info("Step-2.4 Failover end.");
        }

        static function pr_update_htaccess() {
            $settingOptions = new ProtectRemoteSettingOptions();
            $prData = self::pr_data_get();

            $token = $settingOptions->getToken();
            if (!isset($token) || empty($token)) {
                error_log("PR token not found. Please set token in settings page.");
                return;
            }
    
            $result = self::pr_get_white_ip_list($prData);

            if (!$result->is_success) {
                ProtectRemoteLog::pr_log_error("Step-2 Failed to get white ip list.", true);
                self::pr_failover($prData);
                return;
            }
            
            $checkRulesChanges = self::check_change_rules($result, $prData, $settingOptions);
            
            if (!$checkRulesChanges) {
                ProtectRemoteLog::pr_log_warning("Step-3 Rules not changed.", true);
                return;
            }
    
            $contentWithoutPR = self::remove_pr_block_from_htaccess();

            if ($contentWithoutPR == null) {
                ProtectRemoteLog::pr_log_error("Step-4 Failed to remove pr block from htaccess.", true);
                return;
            }
                
            $htaccessContent = file_exists(ProtectRemoteVariables::get_htaccess_file_path()) ? file_get_contents(ProtectRemoteVariables::get_htaccess_file_path()) : null;
    
            if ($htaccessContent != null) {

                $prContent = "# BEGIN Protect Remote\n";
                $prContent .= "<IfModule mod_rewrite.c>\n";
                $prContent .= "RewriteEngine on\n";

                if ($settingOptions->getFlagAdminAjax()) {
                    $prContent .= "RewriteCond %{REQUEST_URI} ^/wp-admin/admin-ajax\.php$ [NC]\n";
                    $prContent .= "RewriteRule ^ - [L]\n";
                }

                if (!empty($settingOptions->getWpLoginPageUrl())) {

                    $prContent .= "RewriteRule ^".$settingOptions->getWpLoginPageUrl()."/?$ wp-login.php [L]\n";
                    $prContent .= "RewriteRule ^wp-login\.php$ - [F,L]\n";
                }                

                $prContent .= "RewriteRule ^pr-access-denied/?$ ".ProtectRemoteVariables::get_access_denied_file_path()." [L]\n";

                if ($settingOptions->getWpCloudflare()) 
                {
                    $ipV4List = $prData->getCloudflareIpV4List();
                    $ipV6List = $prData->getCloudflareIpV6List();

                    $prContent .=  self::pr_prepare_rule($ipV4List);
                    $prContent .=  self::pr_prepare_rule($ipV6List);
                    $prContent .= self::pr_prepare_rule_for_cf($result->ip_list);
                } 
                else 
                {
                    $prContent .=  self::pr_prepare_rule($result->ip_list);
                }

                if ($settingOptions->getFlagWebsite()) 
                {
                    $prContent .= "RewriteCond %{REQUEST_URI} !^/protect-remote-cron-trigger [NC]\n";
                    $prContent .= "RewriteCond %{REQUEST_URI} ^(.*)$ [NC]\n";
                } 
                else 
                {
                    $pages = [];

                    if ($settingOptions->getFlagLoginPage()) {
                        $pages[] = 'wp-login\.php';
                    } 
                    
                    if ($settingOptions->getFlagAdminPanel()) {
                        $pages[] = 'wp-admin';
                    }
                    
                    if ($settingOptions->getFlagHideUsers()) {
                        $pages[] = "author";
                        $pages[] = "\?author";
                        $pages[] = "wp-json/wp/v2/users";
                    }

                    $prContent .= "RewriteCond %{REQUEST_URI} (".implode("|", $pages).") [NC]\n";
                }
                
                $prContent .= "RewriteRule ^(.*)$ /pr-access-denied [R=302,L]\n";
                $prContent .= "</IfModule>\n";
                $prContent .= "# END Protect Remote";
                $prContent .= "\n\n";

                $newContent = $prContent . $contentWithoutPR;
                file_put_contents(ProtectRemoteVariables::get_htaccess_file_path(), $newContent);
                ProtectRemoteLog::pr_log_info("Step-5.1 Successfully updated htaccess file content. Path: ".ProtectRemoteVariables::get_htaccess_file_path());

                self::update_protect_remote_data_json($result->ip_list, $prData,  $settingOptions);
                ProtectRemoteLog::pr_log_info("Step-5.2 Successfully updated protect remote data json file content. Path: ".ProtectRemoteVariables::get_pr_data_json_path());
            } 
            else 
            {
                ProtectRemoteLog::pr_log_error("Step-5 Failed to get htaccess file content. Path: ".ProtectRemoteVariables::get_htaccess_file_path());
            }

            ProtectRemoteLog::pr_log_info("Step-6 finised.", true);
        }

        static function update_protect_remote_data_json($ip_list, $prData, $settingOptions) 
        {
            $prData->setIpList($ip_list);
            $prData->setTryCount(1);
            $prData->setHasBlockInHtaccess(true);
            $prData->setPrFlagLoginPage($settingOptions->getFlagLoginPage());
            $prData->setPrFlagAdminPanel($settingOptions->getFlagAdminPanel());
            $prData->setPrFlagAdminAjax($settingOptions->getFlagAdminAjax());
            $prData->setPrFlagWebsite($settingOptions->getFlagWebSite());
            $prData->setPrWpLoginPageUrl($settingOptions->getWpLoginPageUrl());
            self::pr_data_set($prData);
        }

        static function pr_prepare_rule($ipList) {
            $conditions = "";
            foreach ($ipList as $ip) {
                $conditions .= "RewriteCond %{REMOTE_ADDR} !^".$ip."$\n";
            }
            return $conditions;
        }

        static function pr_prepare_rule_for_cf($ipList) {
            $conditions = "";
            foreach ($ipList as $ip) {
                $conditions .= "RewriteCond %{HTTP:CF-Connecting-IP} !^".$ip."$\n";
            }
            return $conditions;
        }

        static function remove_pr_block_from_htaccess_and_save() {
            $htaccessContent = self::remove_pr_block_from_htaccess();

            if ($htaccessContent != null) 
            {
                file_put_contents(ProtectRemoteVariables::get_htaccess_file_path(), $htaccessContent);
            }
        }
    
        static function remove_pr_block_from_htaccess() {

            $contents = file_exists(ProtectRemoteVariables::get_htaccess_file_path()) ? file(ProtectRemoteVariables::get_htaccess_file_path()) : null;
    
            if ($contents == null) {
                error_log("htaccess file not found: ".ProtectRemoteVariables::get_htaccess_file_path());
                return null;
            }
    
            $newContents = array();
            $delete_mode = false;
    
            foreach ($contents as $row) {
                $trimmed_row = trim($row);
    
                if (strpos($row, '# BEGIN Protect Remote') !== false) {
                    $delete_mode = true;
                }
                
                if (!$delete_mode && $trimmed_row !== '') {
                    $newContents[] = $row;
                }
                
                if (strpos($row, '# END Protect Remote') !== false) {
                    $delete_mode = false;
                }
            }
    
            $newContents = implode('', $newContents);
    
            return $newContents;
        }

        static function check_change_rules($result, $prData, $settingOptions) {
            
            $state = false;

            if ($prData === null || $prData->getIpList() == null || $prData->getHasBlockInHtaccess() == false) {
                ProtectRemoteLog::pr_log_info("Step-3.1 Failed to get pr data.");
                $state = true;
            } 
            
            if ($prData->getIpList() != $result->ip_list) {
                ProtectRemoteLog::pr_log_info("Step-3.2 Ip list changed.");
                $state = true;
            } 

            if ($prData->getPrFlagLoginPage() != $settingOptions->getFlagLoginPage()) {
                ProtectRemoteLog::pr_log_info("Step-3.3 pr_flag_login_page changed.");
                $state = true;
            } 

            if ($prData->getPrFlagAdminPanel() != $settingOptions->getFlagAdminPanel()) {
                ProtectRemoteLog::pr_log_info("Step-3.4 pr_flag_admin_panel changed.");
                $state = true;
            } 

            if ($prData->getPrFlagAdminAjax() != $settingOptions->getFlagAdminAjax()) {
                ProtectRemoteLog::pr_log_info("Step-3.5 pr_flag_admin_ajax changed.");
                $state = true;
            } 

            if ($prData->getPrFlagWebsite() !=  $settingOptions->getFlagWebsite()) {
                ProtectRemoteLog::pr_log_info("Step-3.6 pr_flag_website changed.");
                $state = true;
            } 

            if ($prData->getPrWpLoginPageUrl() != $settingOptions->getWpLoginPageUrl()) {
                ProtectRemoteLog::pr_log_info("Step-3.7 pr_wp_login_page_url changed.");
                $state = true;
            } 
            
            return $state;
        }

        static function pr_data_get() {
            $prData = file_exists(ProtectRemoteVariables::get_pr_data_json_path()) ? file_get_contents(ProtectRemoteVariables::get_pr_data_json_path()) : null;
            
            if ($prData == null) 
                return null;
                            
            return new ProtectRemoteData($prData);
        }

        static function pr_data_set($prData) {

            if (file_exists(ProtectRemoteVariables::get_pr_data_json_path())) {
                file_put_contents(ProtectRemoteVariables::get_pr_data_json_path(), $prData->toJson());
            } else {
                file_put_contents(ProtectRemoteVariables::get_pr_data_json_path(), $prData->toJson());
            }
        }
    }
?>
