<?php 
    class ProtectRemoteCron {

        private static $correlationId;

        private static function generate_correlation_id() {
            $guid = self::generate_guid();
            self::$correlationId = $guid;
        }

        static function get_correlation_id() {
            return self::$correlationId;
        }

        private static function generate_guid() {
            $uniqueId = uniqid('', true);
            $hashedId = md5($uniqueId);
            return substr($hashedId, 0, 8) . '-' . substr($hashedId, 8, 4) . '-' . substr($hashedId, 12, 4) . '-' . substr($hashedId, 16, 4) . '-' . substr($hashedId, 20);
        }

        static function schedule_pr_cron_job() {
            if (!wp_next_scheduled('pr_cron_hook')) {
                wp_schedule_event(time(), 'pr_cron', 'pr_cron_hook');
            }

            ProtectRemoteCron::pr_cron_trigger_page();
        }

        static function pr_cron_trigger_page() {
            $requested_url = esc_url($_SERVER['REQUEST_URI']);
            
            if ($requested_url == "".ProtectRemoteVariables::get_pr_wp_cron_trigger_page_url()."") {
                $message = "<div style='text-align:center;'>This site is protected by <a href='https://protectremote.com' title='Protect Remote'>Protect Remote</a></div>";
                $http_status = 200;
                wp_die($message, 'Protect Remote Trigger Page', array('response' => $http_status));
            }
        }

        static function pr_cron_intervals($schedules) {
            
            $schedules['pr_cron'] = array(
                'interval' => ProtectRemoteVariables::CRON_TIME_SECOND,
                'display'  => __('Protect Remote Cron'), 
            );

            return $schedules;
        }

        static function wp_cron() {
            add_action('init', 'ProtectRemoteCron::schedule_pr_cron_job');
            add_filter('cron_schedules', 'ProtectRemoteCron::pr_cron_intervals');
            add_action('pr_cron_hook', 'ProtectRemote::pr_start');
        }

        static function pr_cron() {

            add_action('init', 'ProtectRemoteCron::pr_cron_trigger_page');
            
            $pr_cron = get_option('pr_cron');
            $now = new DateTime('now', new DateTimeZone('UTC'));
            $last_time = new DateTime('now', new DateTimeZone('UTC'));

            if ($pr_cron == false) 
            {
                add_option('pr_cron', $now);
            } 
            else 
            { 
                $last_time = get_option('pr_cron');
            }

            $dateDiff = $now->diff($last_time);
            $secondDiff = $dateDiff->s;

            if ($secondDiff >= ProtectRemoteVariables::CRON_TIME_SECOND) {
                update_option('pr_cron', $now);
                self::generate_correlation_id();
                ProtectRemoteLog::pr_log_info("Step-0 Cron job started.");
                ProtectRemote::pr_start();
            }
        }
    }
?>