<?php
    class ProtectRemoteData {
        private $ipList = [];
        private $cloudflareIpV4List = [];
        private $cloudflareIpV6List = [];
        private $cloudflareIpV4 = "";
        private $cloudflareIpV6 = "";
        private $tryCount = 0;
        private $hasBlockInHtaccess = false;
        private $prWpLoginPageUrl = "";
        private $prFlagLoginPage = 1;
        private $prFlagAdminPanel = 1;
        private $prFlagAdminAjax = 0;
        private $prFlagWebsite = 0;
        
        public function __construct($jsonData) {

            if ($jsonData) {

                $data = json_decode($jsonData, true);

                if (isset($data['cloudflare_ip_v4']) && is_string($data['cloudflare_ip_v4'])) {
                    $this->cloudflareIpV4 = $data['cloudflare_ip_v4'];
                }

                if (isset($data['cloudflare_ip_v6']) && is_string($data['cloudflare_ip_v6'])) {
                    $this->cloudflareIpV6 = $data['cloudflare_ip_v6'];
                }

                if (isset($data['cloudflare_ip_v4_list']) && is_array($data['cloudflare_ip_v4_list'])) {
                    $this->cloudflareIpV4List = $data['cloudflare_ip_v4_list'];
                }

                if (isset($data['cloudflare_ip_v6_list']) && is_array($data['cloudflare_ip_v6_list'])) {
                    $this->cloudflareIpV6List = $data['cloudflare_ip_v6_list'];
                }

                if (isset($data['ip_list']) && is_array($data['ip_list'])) {
                    $this->ipList = $data['ip_list'];
                }

                if (isset($data['try_count']) && is_numeric($data['try_count'])) {
                    $this->tryCount = $data['try_count'];
                }

                if (isset($data['has_block_in_htaccess']) && is_bool($data['has_block_in_htaccess'])) {
                    $this->hasBlockInHtaccess = $data['has_block_in_htaccess'];
                }

                if (isset($data['pr_wp_login_page_url']) && is_string($data['pr_wp_login_page_url'])) {
                    $this->prWpLoginPageUrl = $data['pr_wp_login_page_url'];
                }

                if (isset($data['pr_flag_login_page']) && is_numeric($data['pr_flag_login_page'])) {
                    $this->prFlagLoginPage = $data['pr_flag_login_page'];
                }

                if (isset($data['pr_flag_admin_panel']) && is_numeric($data['pr_flag_admin_panel'])) {
                    $this->prFlagAdminPanel = $data['pr_flag_admin_panel'];
                }

                if (isset($data['pr_flag_admin_ajax']) && is_numeric($data['pr_flag_admin_ajax'])) {
                    $this->prFlagAdminAjax = $data['pr_flag_admin_ajax'];
                }

                if (isset($data['pr_flag_website']) && is_numeric($data['pr_flag_website'])) {
                    $this->prFlagWebsite = $data['pr_flag_website'];
                }
            }
        }

        public function getJsonData() {
            $data = [
                'ip_list' => $this->ipList,
                'cloudflare_ip_v4_list' => $this->cloudflareIpV4List,
                'cloudflare_ip_v6_list' => $this->cloudflareIpV6List,
                'cloudflare_ip_v4' => $this->cloudflareIpV4,
                'cloudflare_ip_v6' => $this->cloudflareIpV6,
                'try_count' => $this->tryCount,
                'has_block_in_htaccess' => $this->hasBlockInHtaccess,
                'pr_wp_login_page_url' => $this->prWpLoginPageUrl,
                'pr_flag_login_page' => $this->prFlagLoginPage,
                'pr_flag_admin_panel' => $this->prFlagAdminPanel,
                'pr_flag_admin_ajax' => $this->prFlagAdminAjax,
                'pr_flag_website' => $this->prFlagWebsite,
            ];
        
            return json_encode($data);
        }        

        public function toJson() {
            $data = [
                'ip_list' => $this->ipList,
                'cloudflare_ip_v4_list' => $this->cloudflareIpV4List,
                'cloudflare_ip_v6_list' => $this->cloudflareIpV6List,
                'cloudflare_ip_v4' => $this->cloudflareIpV4,
                'cloudflare_ip_v6' => $this->cloudflareIpV6,
                'try_count' => $this->tryCount,
                'has_block_in_htaccess' => $this->hasBlockInHtaccess,
                'pr_wp_login_page_url' => $this->prWpLoginPageUrl,
                'pr_flag_login_page' => $this->prFlagLoginPage,
                'pr_flag_admin_panel' => $this->prFlagAdminPanel,
                'pr_flag_admin_ajax' => $this->prFlagAdminAjax,
                'pr_flag_website' => $this->prFlagWebsite,
            ];
    
            return json_encode($data);
        }

        public function getPrWpLoginPageUrl() {
            return $this->prWpLoginPageUrl;
        }

        public function setPrWpLoginPageUrl($prWpLoginPageUrl) {
            $this->prWpLoginPageUrl = $prWpLoginPageUrl;
        }

        public function getPrFlagLoginPage() {
            return $this->prFlagLoginPage;
        }

        public function setPrFlagLoginPage($prFlagLoginPage) {
            if (is_numeric($prFlagLoginPage)) {
                $this->prFlagLoginPage = $prFlagLoginPage;
            }
        }

        public function getPrFlagAdminPanel() {
            return $this->prFlagAdminPanel;
        }

        public function setPrFlagAdminPanel($prFlagAdminPanel) {
            if (is_numeric($prFlagAdminPanel)) {
                $this->prFlagAdminPanel = $prFlagAdminPanel;
            }
        }

        public function getPrFlagAdminAjax() {
            return $this->prFlagAdminAjax;
        }

        public function setPrFlagAdminAjax($prFlagAdminAjax) {
            if (is_numeric($prFlagAdminAjax)) {
                $this->prFlagAdminAjax = $prFlagAdminAjax;
            }
        }

        public function getPrFlagWebsite() {
            return $this->prFlagWebsite;
        }

        public function setPrFlagWebsite($prFlagWebsite) {
            if (is_numeric($prFlagWebsite)) {
                $this->prFlagWebsite = $prFlagWebsite;
            }
        }

        public function getIpList() {
            return $this->ipList;
        }

        public function setIpList($ipList) {
            if (is_array($ipList)) {
                $this->ipList = $ipList;
            }
        }
        
        public function getCloudflareIpV4() {
            return $this->cloudflareIpV4;
        }

        public function setCloudflareIpV4($guid) {
            if (is_string($guid)) {
                $this->cloudflareIpV4 = $guid;
            }
        }

        public function getCloudflareIpV4List() {
            return $this->cloudflareIpV4List;
        }

        public function setCloudflareIpV4List($list) {
            if (is_array($list)) {
                $this->cloudflareIpV4List = $list;
            }
        }

        public function getCloudflareIpV6() {
            return $this->cloudflareIpV6;
        }

        public function setCloudflareIpV6($guid) {
            if (is_string($guid)) {
                $this->cloudflareIpV6 = $guid;
            }
        }

        public function getCloudflareIpV6List() {
            return $this->cloudflareIpV6List;
        }

        public function setCloudflareIpV6List($list) {
            if (is_array($list)) {
                $this->cloudflareIpV6List = $list;
            }
        }

        public function getTryCount() {
            return $this->tryCount;
        }

        public function setTryCount($tryCount) {
            if (is_numeric($tryCount)) {
                $this->tryCount = $tryCount;
            }
        }

        public function getHasBlockInHtaccess() {
            return $this->hasBlockInHtaccess;
        }

        public function setHasBlockInHtaccess($hasBlockInHtaccess) {
            if (is_bool($hasBlockInHtaccess)) {
                $this->hasBlockInHtaccess = $hasBlockInHtaccess;
            }
        }
    }
?>