<?php
    class ProtectRemoteSettingOptions {
        private $footer_link = true;
        private $token = "";
        private $flag_login_page = true;
        private $flag_admin_panel = true;
        private $flag_admin_ajax = false;
        private $flag_website = false;
        private $flag_hide_users = true;
        private $wp_login_page_url = "";
        private $wp_admin_page_url = "";
        private $wp_log_mode = false;
        private $wp_cloudflare = false;
        private $wp_footer_keyword_id = -1;

        public function __construct() {

            $data = get_option('pr_setting_options');

            if ($data) {
                $data = json_decode($data, true);
            }

            if (isset($data['footer_link']) && is_bool($data['footer_link'])) {
                $this->footer_link = $data['footer_link'];
            }

            if (isset($data['token']) && is_string($data['token'])) {
                $this->token = $data['token'];
            }

            if (isset($data['flag_login_page']) && is_bool($data['flag_login_page'])) {
                $this->flag_login_page = $data['flag_login_page'];
            }

            if (isset($data['flag_admin_panel']) && is_bool($data['flag_admin_panel'])) {
                $this->flag_admin_panel = $data['flag_admin_panel'];
            }

            if (isset($data['flag_admin_panel']) && is_bool($data['flag_admin_panel'])) {
                $this->flag_admin_panel = $data['flag_admin_panel'];
            }

            if (isset($data['flag_admin_ajax']) && is_bool($data['flag_admin_ajax'])) {
                $this->flag_admin_ajax = $data['flag_admin_ajax'];
            }

            if (isset($data['flag_website']) && is_bool($data['flag_website'])) {
                $this->flag_website = $data['flag_website'];
            }
            
            if (isset($data['flag_hide_users']) && is_bool($data['flag_hide_users'])) {
                $this->flag_hide_users = $data['flag_hide_users'];
            }

            if (isset($data['wp_login_page_url']) && is_string($data['wp_login_page_url'])) {
                $this->wp_login_page_url = $data['wp_login_page_url'];
            }

            if (isset($data['wp_admin_page_url']) && is_string($data['wp_admin_page_url'])) {
                $this->wp_admin_page_url = $data['wp_admin_page_url'];
            }

            if (isset($data['wp_log_mode']) && is_bool($data['wp_log_mode'])) {
                $this->wp_log_mode = $data['wp_log_mode'];
            }

            if (isset($data['wp_cloudflare']) && is_bool($data['wp_cloudflare'])) {
                $this->wp_cloudflare = $data['wp_cloudflare'];
            }

            if (isset($data['wp_footer_keyword_id']) && is_numeric($data['wp_footer_keyword_id'])) {
                $this->wp_footer_keyword_id = $data['wp_footer_keyword_id'];
            }
        }

        public function saveChanges() {
            $data = $this->toJson();
            update_option("pr_setting_options", $data);
        }
    
        public function toJson() {
            $data = [
                'footer_link' => $this->footer_link,
                'token' => $this->token,
                'flag_login_page' => $this->flag_login_page,
                'flag_admin_panel' => $this->flag_admin_panel,
                'flag_admin_ajax' => $this->flag_admin_ajax,
                'flag_website' => $this->flag_website,
                'flag_hide_users' => $this->flag_hide_users,
                'wp_login_page_url' => $this->wp_login_page_url,
                'wp_admin_page_url' => $this->wp_admin_page_url,
                'wp_log_mode' => $this->wp_log_mode,
                'wp_cloudflare' => $this->wp_cloudflare,
                'wp_footer_keyword_id' => $this->wp_footer_keyword_id
            ];
    
            return json_encode($data);
        }

        // Footer Link Property
        public function getFooterLink() {
            return $this->footer_link;
        }

        public function setFooterLink($footer_link) {
            $footer_link = $footer_link == "1" ? true : false;
            $this->footer_link = $footer_link;
        }

        // Token Property
        public function getToken() {
            return $this->token;
        }
    
        public function setToken($token) {
            $this->token = $token;
        }
    
        // Flag LoginPage Property
        public function getFlagLoginPage() {
            return $this->flag_login_page;
        }
    
        public function setFlagLoginPage($flagLoginPage) {
            $flagLoginPage = $flagLoginPage == "1" ? true : false;
            $this->flag_login_page = $flagLoginPage;
        }
    
        // Flag AdminPanel Property
        public function getFlagAdminPanel() {
            return $this->flag_admin_panel;
        }
    
        public function setFlagAdminPanel($flagAdminPanel) {
            $flagAdminPanel = $flagAdminPanel == "1" ? true : false;
            $this->flag_admin_panel = $flagAdminPanel;
        }
    
        // Flag AdminAjax Property
        public function getFlagAdminAjax() {
            return $this->flag_admin_ajax;
        }
    
        public function setFlagAdminAjax($flagAdminAjax) {
            $flagAdminAjax = $flagAdminAjax == "1" ? true : false;
            $this->flag_admin_ajax = $flagAdminAjax;
        }
    
        // Flag Website Property
        public function getFlagWebsite() {
            return $this->flag_website;
        }
    
        public function setFlagWebsite($flagWebsite) {
            $flagWebsite = $flagWebsite == "1" ? true : false;
            $this->flag_website = $flagWebsite;
        }

        // Flag Hide Users Property
        public function getFlagHideUsers() {
            return $this->flag_hide_users;
        }
    
        public function setFlagHideUsers($flagHideUsers) {
            $flagHideUsers = $flagHideUsers == "1" ? true : false;
            $this->flag_hide_users = $flagHideUsers;
        }
    
        // WpLoginPageUrl Property
        public function getWpLoginPageUrl() {
            return $this->wp_login_page_url;
        }
    
        public function setWpLoginPageUrl($wpLoginPageUrl) {
            $this->wp_login_page_url = $wpLoginPageUrl;
        }
    
        // WpAdminPageUrl Property
        public function getWpAdminPageUrl() {
            return $this->wp_admin_page_url;
        }
    
        public function setWpAdminPageUrl($wpAdminPageUrl) {
            $this->wp_admin_page_url = $wpAdminPageUrl;
        }
    
        // WpLogMode Property
        public function getWpLogMode() {
            return $this->wp_log_mode;
        }
    
        public function setWpLogMode($wpLogMode) {
            $wpLogMode = $wpLogMode == "True" ? true : false;
            $this->wp_log_mode = $wpLogMode;
        }

        // WpCloudflare Property
        public function getWpCloudflare() {
            return $this->wp_cloudflare;
        }
    
        public function setWpCloudflare($wpCloudflare) {
            $wpCloudflare = $wpCloudflare == "True" ? true : false;
            $this->wp_cloudflare = $wpCloudflare;
        }

        public function getWpFooterKeywordId() {
            return $this->wp_footer_keyword_id;
        }

        public function setWpFooterKeywordId($wpFooterKeywordId) {
            $this->wp_footer_keyword_id = $wpFooterKeywordId;
        }
    }    
?>